<?php
namespace PrestaShop\Module\PolarisPrestaConnector;

/**
 * Synchronisation des commandes
 */
class SyncerOrders
{
    /**
     * Initialisé ?
     */
    private bool $inited = false;

    /**
     * Synchroniseur
     * 
     */
    private Syncer $syncer;

    /**
     * Constructeur
     * 
     * @param Syncer $syncer Synchroniseur
     */
    public function __construct(Syncer $syncer)
    {
        $this->syncer = $syncer;
        $this->init();
    }

    /**
     * Initialisation
     * 
     */
    private function init()
    {
        if ($this->inited)
            return;

        // On charge l'état d'exportation des commandes ou on le crée s'il n'existe pas !
        $cfg = $this->syncer->module->getCfg();

        $export_state_id = $cfg->get(SyncConfiguration::CFG_ORDER_EXPORT_STATE);

        // On vérifie qu'on a bien un état d'exportation
        // On charge l'état d'exportation des commandes ou on le crée s'il n'existe pas !
        if ($export_state_id)
        {
            $export_state = new \OrderState($export_state_id);
            if (!\Validate::isLoadedObject($export_state))
                $export_state_id = null;
        }

        if (!$export_state_id)
        {
            // Création de l'état d'exportation des commandes
            $export_state = new \OrderState();
            $export_state->name = array();
            foreach (\Language::getLanguages() as $lang)
                $export_state->name[$lang['id_lang']] = 'Exportée vers '.$this->syncer->module->getBridge()->getId();
            $export_state->send_email = false;
            $export_state->color = '#fffcb5';
            $export_state->hidden = true;
            $export_state->delivery = false;
            $export_state->logable = true;
            $export_state->invoice = true;
            $export_state->unremovable = true;
            $export_state->paid = true;
            $export_state->module_name = $this->syncer->module->name;
            $export_state->deleted = false;
            $export_state->add();

            $export_state_id = $export_state->id;
            // Sauvegarde dans la configuration
            $cfg->set(SyncConfiguration::CFG_ORDER_EXPORT_STATE, $export_state_id);
        }

        // On vérifie qu'on a bien un état d'erreur d'exportation
        $error_export_state_id = $cfg->get(SyncConfiguration::CFG_ORDER_ERROR_EXPORT_STATE);
        if ($error_export_state_id)
        {
            $export_state = new \OrderState($error_export_state_id);
            if (!\Validate::isLoadedObject($export_state))
                $error_export_state_id = null;
        }

        if (!$error_export_state_id)
        {
            // Création de l'état d'exportation des commandes
            $export_state = new \OrderState();
            $export_state->name = array();
            foreach (\Language::getLanguages() as $lang)
                $export_state->name[$lang['id_lang']] = 'Erreur d\'export vers '.$this->syncer->module->getBridge()->getId();
            $export_state->send_email = false;
            $export_state->color = '#ff6661';
            $export_state->hidden = true;
            $export_state->delivery = false;
            $export_state->logable = true;
            $export_state->invoice = true;
            $export_state->paid = true;
            $export_state->unremovable = true;
            $export_state->module_name = $this->syncer->module->name;
            $export_state->deleted = false;
            $export_state->add();

            $export_state_id = $export_state->id;
            // Sauvegarde dans la configuration
            $cfg->set(SyncConfiguration::CFG_ORDER_ERROR_EXPORT_STATE, $export_state_id);
        }

        if (!$cfg->get(SyncConfiguration::CFG_SHIPPING_COST_REF))
            throw new \Exception("Référence pour les frais de port non défini : merci de préciser la référence ".$this->syncer->module->getBridge()->getId()." du produit désignant les frais de port !");

        if (!$cfg->get(SyncConfiguration::CFG_WRAPPING_COST_REF))
            throw new \Exception("Référence pour les emballages non défini : merci de préciser la référence ".$this->syncer->module->getBridge()->getId()." du produit désignant les frais d'emballage !");

        $this->inited = true;
    }

    /**
     * Synchronisation des stocks
     */
    public function sync()
    {
        $bridge = $this->syncer->bridge;
        $db = \Db::getInstance();

        $startTime = microtime(true);
        $nbOrders = 0;

        // Etat "exportée"
        $cfg = $this->syncer->module->getCfg();
        $export_state_id = $cfg->get(SyncConfiguration::CFG_ORDER_EXPORT_STATE);
        $error_export_state_id = $cfg->get(SyncConfiguration::CFG_ORDER_ERROR_EXPORT_STATE);

        $max_age = $cfg->get(SyncConfiguration::CFG_ORDER_MAX_TIME);
        if (!$max_age)
            $max_age = 30;

        $order_barrier = $cfg->get(SyncConfiguration::CFG_ORDER_BARRIER);
        if (!$order_barrier)
        {
            // La barrière n'est pas encore positionnée, pour éviter les accidents, on la positionne maintenant sur le plus haut id de commande !
            $sql = 'SELECT MAX(id_order) AS max_order FROM '._DB_PREFIX_.'orders';
            $order_barrier = $db->getValue($sql);
            if (!$order_barrier)
                $order_barrier = 0;
            $cfg->set(SyncConfiguration::CFG_ORDER_BARRIER, $order_barrier);
        }
        
        // On liste tous les id de commandes qui n'ont jamais eu l'état "exportée" dans l'historique des états de commandes
        // Mais qui sont passée par un état paiement accepté
        $sql = 'SELECT DISTINCT o.reference FROM '._DB_PREFIX_.'order_history AS os LEFT JOIN '._DB_PREFIX_.'orders AS o USING (id_order) WHERE 
                    os.date_add > DATE_SUB(NOW(), INTERVAL '.$max_age.' DAY) AND
                    os.id_order > '.$order_barrier.' AND
                    os.id_order_state IN (SELECT id_order_state FROM '._DB_PREFIX_.'order_state WHERE logable = 1 AND paid = 1) AND
                    os.id_order NOT IN (SELECT id_order FROM '._DB_PREFIX_.'order_history WHERE id_order_state = '.$export_state_id.' OR id_order_state = '.$error_export_state_id.')
                    ORDER BY os.id_order ASC
                    ';
        $orders = $db->executeS($sql);
        if ($orders)
        {
            foreach ($orders as $order)
            {
                try
                {
                    if ($this->exportOrder($order['reference']))
                    {
                        // La commande a été synchronisée : on le marque dans le journal !
                        $this->syncer->audit("Commande ".$order['reference']." exportée !");
                    }
                }
                catch (\Exception $e)
                {
                    $msg = "Erreur lors de la synchronisation de la commande ".$order['reference']. " : ".$e->getMessage();
                    $this->syncer->audit($msg);
                    // On met également le message dans le journal de prestashop en mode erreur, en liant à la commande
                    \PrestaShopLogger::addLog($msg, 3, null, $this->syncer->module->name); 
                }
                $nbOrders++;
            }
        }
        else if ($orders === FALSE)
            throw new \Exception('Erreur à la récupération des commandes à synchroniser : '.$db->getMsgError());
        else
            $this->syncer->audit("Aucune commande à synchroniser");
        
        $stopTime = microtime(true);
        if ($nbOrders > 0)
            $this->syncer->audit(sprintf("Synchronisation de %d commandes effectuée en %.2f secondes", $nbOrders, $stopTime - $startTime));    
    }

    /**
     * Synchronisation de la commande précisée
     * 
     * @param string $reference Référence de la commande à exporter
     * @return bool Vrai si la commande a été exportée, faux sinon
     */
    public function exportOrder(string $reference) : bool
    {        
        $cfg = $this->syncer->module->getCfg();

        $psOrders = null;
        try
        {
            // On charge les commandes concernées
            $psOrders = \Order::getByReference($reference)->getAll()->getResults();
            if (!$psOrders)
                throw new \Exception('Commande '.$reference.' introuvable !');
            
            // La suite est simple, on appelle le bridge pour exporter la commande et le client lié
            // Puis on exporte la commande
            $exportWrn = $this->syncer->bridge->pushOrdersAsOneSale($psOrders);
            if (!$exportWrn)
            {
                // On met la commande en exportée
                foreach ($psOrders as $psOrder)
                {
                    $history = new \OrderHistory();
                    $history->id_order = $psOrder->id;
                    $history->id_order_state = $cfg->get(SyncConfiguration::CFG_ORDER_EXPORT_STATE);
                    $history->id_employee = 0;
                    $history->save();
                }

                return true;
            }
            else
            {
                // On signale dans le journal d'audit que la commande a été ignorée et la raison
                $this->syncer->audit("Commande ".$reference." ignorée : ".$exportWrn);
                return false;
            }            
        }
        catch (\Exception $e)
        {
            // On met les commandes en erreur, sans déclencher de hook
            if ($psOrders)
            {
                foreach ($psOrders as $psOrder)
                {
                    $history = new \OrderHistory();
                    $history->id_order = $psOrder->id;
                    $history->id_order_state = $cfg->get(SyncConfiguration::CFG_ORDER_ERROR_EXPORT_STATE);
                    $history->id_employee = 0;
                    $history->save();
                }
            }

            throw $e;   
        }
    }
}